"""public API of Flask-Reuploaded

isort:skip_file
"""
# This huge list of imports is kept on purpose,
# as `Flask-Uploads` provided them as public API,
# and `Flask-Reuploaded` tries to stay compatible.
from .exceptions import UploadNotAllowed

from .extensions import ALL
from .extensions import AllExcept
from .extensions import TEXT
from .extensions import DOCUMENTS
from .extensions import IMAGES
from .extensions import AUDIO
from .extensions import DATA
from .extensions import SCRIPTS
from .extensions import ARCHIVES
from .extensions import SOURCE
from .extensions import EXECUTABLES
from .extensions import DEFAULTS
from .extensions import extension
from .extensions import lowercase_ext


from .flask_uploads import UploadConfiguration
from .flask_uploads import UploadSet
from .flask_uploads import addslash
from .flask_uploads import configure_uploads
from .flask_uploads import config_for_set
from .test_helper import TestingFileStorage

__all__ = [
    "TestingFileStorage",
    "UploadConfiguration",
    "UploadSet",
    "addslash",
    "configure_uploads",
    "extension",
    "lowercase_ext",
    "config_for_set",
    "ALL",
    "AllExcept",
    "TEXT",
    "DOCUMENTS",
    "IMAGES",
    "AUDIO",
    "DATA",
    "SCRIPTS",
    "ARCHIVES",
    "SOURCE",
    "EXECUTABLES",
    "DEFAULTS",
    "UploadNotAllowed",
]



"""Extension presets and extension configuration."""
import os
from typing import Iterable

# This contains archive and compression formats (.gz, .bz2, .zip, .tar,
# .tgz, .txz, and .7z).
ARCHIVES = tuple('gz bz2 zip tar tgz txz 7z'.split())


# This contains audio file types (.wav, .mp3, .aac, .ogg, .oga, and .flac).
AUDIO = tuple('wav mp3 aac ogg oga flac'.split())

# This is for structured data files (.csv, .ini, .json, .plist, .xml, .yaml,
# and .yml).
DATA = tuple('csv ini json plist xml yaml yml'.split())

# This contains various office document formats (.rtf, .odf, .ods, .gnumeric,
# .abw, .doc, .docx, .xls, .xlsx and .pdf). Note that the macro-enabled
# versions of Microsoft Office 2007 files are not included.
DOCUMENTS = tuple('rtf odf ods gnumeric abw doc docx xls xlsx pdf'.split())

# This contains shared libraries and executable files (.so, .exe and .dll).
# Most of the time, you will not want to allow this - it's better suited for
# use with `AllExcept`.
EXECUTABLES = tuple('so exe dll'.split())

# This contains basic image types that are viewable from most browsers (.jpg,
# .jpe, .jpeg, .png, .gif, .svg, .bmp and .webp).
IMAGES = tuple('jpg jpe jpeg png gif svg bmp webp'.split())

# This contains various types of scripts (.js, .php, .pl, .py .rb, and .sh).
# If your Web server has PHP installed and set to auto-run, you might want to
# add ``php`` to the DENY setting.
SCRIPTS = tuple('js php pl py rb sh'.split())

# This contains nonexecutable source files - those which need to be
# compiled or assembled to binaries to be used. They are generally safe to
# accept, as without an existing RCE vulnerability, they cannot be compiled,
# assembled, linked, or executed. Supports C, C++, Ada, Rust, Go (Golang),
# FORTRAN, D, Java, C Sharp, F Sharp (compiled only), COBOL, Haskell, and
# assembly.
SOURCE = tuple(
    (
        'c cpp c++ h hpp h++ cxx hxx hdl '  # C/C++
        'ada '  # Ada
        'rs '  # Rust
        'go '  # Go
        'f for f90 f95 f03 '  # FORTRAN
        'd dd di '  # D
        'java '  # Java
        'hs '  # Haskell
        'cs '  # C Sharp
        'fs '  # F Sharp compiled source (NOT .fsx, which is interactive-ready)
        'cbl cob '  # COBOL
        'asm s '  # Assembly
    ).split()
)

# This just contains plain text files (.txt).
TEXT = ('txt',)

# The default allowed extensions - `TEXT`, `DOCUMENTS`, `DATA`, and `IMAGES`.
DEFAULTS = TEXT + DOCUMENTS + IMAGES + DATA


class All:
    """
    This type can be used to allow all extensions. There is a predefined
    instance named `ALL`.
    """
    def __contains__(self, item: str) -> bool:
        return True


#: This "contains" all items. You can use it to allow all extensions to be
#: uploaded.
ALL = All()


class AllExcept:
    """
    This can be used to allow all file types except certain ones. For example,
    to ban .exe and .iso files, pass::

        AllExcept(('exe', 'iso'))

    to the `UploadSet` constructor as `extensions`. You can use any container,
    for example::

        AllExcept(SCRIPTS + EXECUTABLES)
    """
    def __init__(self, items: Iterable[str]) -> None:
        self.items = items

    def __contains__(self, item: str) -> bool:
        return item not in self.items


def extension(filename: str) -> str:
    ext = os.path.splitext(filename)[1]
    if ext.startswith('.'):
        # os.path.splitext retains . separator
        ext = ext[1:]
    return ext


def lowercase_ext(filename: str) -> str:
    """
    This is a helper used by UploadSet.save to provide lowercase extensions for
    all processed files, to compare with configured extensions in the same
    case.

    .. versionchanged:: 0.1.4
       Filenames without extensions are no longer lowercased, only the
       extension is returned in lowercase, if an extension exists.

    :param filename: The filename to ensure has a lowercase extension.
    """
    if '.' in filename:
        main, ext = os.path.splitext(filename)
        return main + ext.lower()
    # For consistency with os.path.splitext,
    # do not treat a filename without an extension as an extension.
    # That is, do not return filename.lower().
    return filename



import os

from flask import Flask
from flask_uploads import ALL
from flask_uploads import IMAGES
from flask_uploads import UploadSet
from flask_uploads import configure_uploads

SIMPLE_PICTURE = (
    b"\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x05\x00\x00\x00\x05"
    b"\x08\x02\x00\x00\x00\x02\r\xb1\xb2\x00\x00\x00\x0cIDAT\x08\xd7c`\xa0."
    b'\x00\x00\x00P\x00\x01"\x13\xe8u\x00\x00\x00\x00IEND\xaeB`\x82'
)


def test_autoserve_works_no_longer_without_configuration(tmp_path):
    """this *feature* was pretty much undocumented

    As it could lead to unwanted data disclosure,
    it was deactivated by default with Flask-Reuploaded version 1.0
    """
    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    photos = UploadSet("photos", IMAGES)
    app.config["UPLOADED_PHOTOS_DEST"] = str(image_directory)
    app.config["SECRET_KEY"] = os.urandom(24)
    configure_uploads(app, photos)

    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")
    assert response.status == "404 NOT FOUND"


def test_autoserve_needs_to_be_activated(tmp_path):
    """this *feature* was pretty much undocumented

    As it could lead to unwanted data disclosure,
    it was deactivated with Flask-Reuploaded version 1.0
    It has to be activated by e.g.
    ``app.config["UPLOADS_AUTOSERVE"] = True``
    """
    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    photos = UploadSet("photos", IMAGES)
    app.config["UPLOADED_PHOTOS_DEST"] = str(image_directory)
    app.config["SECRET_KEY"] = os.urandom(24)
    app.config["UPLOADS_AUTOSERVE"] = True
    configure_uploads(app, photos)
    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")
    assert response.status == "200 OK"


def test_autoserve_does_not_work_for_non_existing_upload_set(tmp_path):
    """
    ie here a `files` UploadSet is defined,
    but the client tries to access `photos`

    Although the file is present,
    it cannot be accessed via the `photos` route.
    """

    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    files = UploadSet("files", ALL)
    app.config["UPLOADED_FILES_DEST"] = str(image_directory)
    app.config["UPLOADS_AUTOSERVE"] = True
    app.config["SECRET_KEY"] = os.urandom(24)
    configure_uploads(app, files)

    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")

    assert response.status == "404 NOT FOUND"


def test_autoserve_gets_deactivated_when_configuring_url_with_empty_string(
        tmp_path):
    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    photos = UploadSet("photos", IMAGES)
    app.config["UPLOADED_PHOTOS_DEST"] = str(image_directory)
    app.config["SECRET_KEY"] = os.urandom(24)

    # this deactivates autoserve
    app.config["UPLOADED_PHOTOS_URL"] = ""

    configure_uploads(app, photos)

    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")

    assert response.status == "404 NOT FOUND"


def test_autoserve_gets_deactivated_when_configuring_uploaded_url(tmp_path):
    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    photos = UploadSet("photos", IMAGES)
    app.config["UPLOADED_PHOTOS_DEST"] = str(image_directory)
    app.config["SECRET_KEY"] = os.urandom(24)

    # configuring this deactivates autoserve
    app.config["UPLOADED_PHOTOS_URL"] = "https://example.com/images"

    configure_uploads(app, photos)

    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")

    assert response.status == "404 NOT FOUND"


def test_autoserve_gets_deactivated_when_set_autoserve_to_false(tmp_path):
    static = tmp_path / "static"
    static.mkdir()
    image_directory = static / "images"
    image_directory.mkdir()
    p = image_directory / "snow.jpg"
    p.write_bytes(SIMPLE_PICTURE)

    app = Flask(__name__)
    photos = UploadSet("photos", IMAGES)
    app.config["UPLOADED_PHOTOS_DEST"] = str(image_directory)
    app.config["SECRET_KEY"] = os.urandom(24)

    # configuring this deactivates autoserve
    # this is the default behavior since Flask-Reuploaded 1.0
    app.config["UPLOADS_AUTOSERVE"] = False

    configure_uploads(app, photos)

    with app.test_client() as client:
        response = client.get("/_uploads/photos/snow.jpg")

    assert response.status == "404 NOT FOUND"



"""
:copyright: 2010 Matthew "LeafStorm" Frazier
:copyright: 2019-2020 Jürgen Gmach <juergen.gmach@googlemail.com>
:license:   MIT/X11, see LICENSE for details
"""

import os
import os.path

import pytest
from flask import Flask
from flask import url_for
from flask_uploads import ALL
from flask_uploads import AllExcept
from flask_uploads import TestingFileStorage
from flask_uploads import UploadConfiguration
from flask_uploads import UploadNotAllowed
from flask_uploads import UploadSet
from flask_uploads import addslash
from flask_uploads import config_for_set
from flask_uploads import configure_uploads
from flask_uploads import extension
from flask_uploads import lowercase_ext


class TestMiscellaneous:
    def test_tfs(self):
        tfs = TestingFileStorage(filename='foo.bar')
        assert tfs.filename == 'foo.bar'
        assert tfs.name is None
        assert tfs.saved is None
        tfs.save('foo_bar.txt')
        assert tfs.saved == 'foo_bar.txt'

    def test_extension(self):
        assert extension('foo.txt') == 'txt'
        assert extension('foo') == ''
        assert extension('archive.tar.gz') == 'gz'
        assert extension('audio.m4a') == 'm4a'

    def test_lowercase_ext(self):
        assert lowercase_ext('foo.txt') == 'foo.txt'
        assert lowercase_ext('FOO.TXT') == 'FOO.txt'
        assert lowercase_ext('foo') == 'foo'
        assert lowercase_ext('FOO') == 'FOO'
        assert lowercase_ext('archive.tar.gz') == 'archive.tar.gz'
        assert lowercase_ext('ARCHIVE.TAR.GZ') == 'ARCHIVE.TAR.gz'
        assert lowercase_ext('audio.m4a') == 'audio.m4a'
        assert lowercase_ext('AUDIO.M4A') == 'AUDIO.m4a'

    def test_addslash(self):
        assert (addslash('http://localhost:4000') ==
                'http://localhost:4000/')
        assert (addslash('http://localhost/uploads') ==
                'http://localhost/uploads/')
        assert (addslash('http://localhost:4000/') ==
                'http://localhost:4000/')
        assert (addslash('http://localhost/uploads/') ==
                'http://localhost/uploads/')

    def test_custom_iterables(self):
        assert 'txt' in ALL
        assert 'exe' in ALL
        ax = AllExcept(['exe'])
        assert 'txt' in ax
        assert 'exe' not in ax


Config = UploadConfiguration


class TestConfiguration:
    def setup(self):
        self.app = Flask(__name__)

    def teardown(self):
        del self.app

    def configure(self, *sets, **options):
        self.app.config.update(options)
        configure_uploads(self.app, sets)
        return self.app.upload_set_config

    def test_compare_upload_configurations(self):
        """UploadConfigurations are only comparable to UploadConfigurations"""
        rv = Config("/var/files", "http://localhost").__eq__("abc")
        assert rv is NotImplemented

    def test_manual(self):
        f, p = UploadSet('files'), UploadSet('photos')
        setconfig = self.configure(
            f, p,
            UPLOADED_FILES_DEST='/var/files',
            UPLOADED_FILES_URL='http://localhost:6001/',
            UPLOADED_PHOTOS_DEST='/mnt/photos',
            UPLOADED_PHOTOS_URL='http://localhost:6002/'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config('/var/files', 'http://localhost:6001/')
        assert photo_conf == Config('/mnt/photos', 'http://localhost:6002/')

    def test_selfserve(self):
        f, p = UploadSet('files'), UploadSet('photos')
        setconfig = self.configure(
            f, p,
            UPLOADED_FILES_DEST='/var/files',
            UPLOADED_PHOTOS_DEST='/mnt/photos'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config('/var/files', None)
        assert photo_conf == Config('/mnt/photos', None)

    def test_defaults(self):
        f, p = UploadSet('files'), UploadSet('photos')
        setconfig = self.configure(
            f, p,
            UPLOADS_DEFAULT_DEST='/var/uploads',
            UPLOADS_DEFAULT_URL='http://localhost:6000/'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config(
            '/var/uploads/files', 'http://localhost:6000/files/')
        assert photo_conf == Config(
            '/var/uploads/photos', 'http://localhost:6000/photos/')

    def test_default_selfserve(self):
        f, p = UploadSet('files'), UploadSet('photos')
        setconfig = self.configure(
            f, p,
            UPLOADS_DEFAULT_DEST='/var/uploads'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config('/var/uploads/files', None)
        assert photo_conf == Config('/var/uploads/photos', None)

    def test_mixed_defaults(self):
        f, p = UploadSet('files'), UploadSet('photos')
        setconfig = self.configure(
            f, p,
            UPLOADS_DEFAULT_DEST='/var/uploads',
            UPLOADS_DEFAULT_URL='http://localhost:6001/',
            UPLOADED_PHOTOS_DEST='/mnt/photos',
            UPLOADED_PHOTOS_URL='http://localhost:6002/'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config(
            '/var/uploads/files', 'http://localhost:6001/files/')
        assert photo_conf == Config('/mnt/photos', 'http://localhost:6002/')

    def test_default_destination_callable(self):
        f = UploadSet('files', default_dest=lambda app: os.path.join(
            app.config['INSTANCE'], 'files'
        ))
        p = UploadSet('photos')
        setconfig = self.configure(
            f, p,
            INSTANCE='/home/me/webapps/thisapp',
            UPLOADED_PHOTOS_DEST='/mnt/photos',
            UPLOADED_PHOTOS_URL='http://localhost:6002/'
        )
        file_conf, photo_conf = setconfig['files'], setconfig['photos']
        assert file_conf == Config('/home/me/webapps/thisapp/files', None)
        assert photo_conf == Config('/mnt/photos', 'http://localhost:6002/')


class TestPreconditions:
    def test_filenames(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        namepairs = (
            ('foo.txt', True),
            ('boat.jpg', True),
            ('warez.exe', False)
        )
        for name, result in namepairs:
            tfs = TestingFileStorage(filename=name)
            assert uset.file_allowed(tfs, name) is result

    def test_underscores_are_not_allowed_for_names_in_upload_sets(self):
        with pytest.raises(ValueError):
            UploadSet("__not__allowed__")

    def test_default_extensions(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        extpairs = (('txt', True), ('jpg', True), ('exe', False))
        for ext, result in extpairs:
            assert uset.extension_allowed(ext) is result

    def test_app_is_properly_configured(self):
        """`configure_uploads` needs to be called

        otherwise a RuntimeError gets raised
        """
        app = Flask(__name__)
        files = UploadSet("files", ALL)
        # In order to cause a RuntimeError the application must not be properly
        # configured, that is, the following line must not be executed:
        # configure_uploads(app, files)
        with app.test_request_context(environ_base={'HTTP_NAME': 'value'}):
            with pytest.raises(RuntimeError) as excinfo:
                files.config
        expected = "The application is not properly configured. "
        expected += "Please make sure to use `configure_uploads`."
        assert str(excinfo.value) == expected


class TestSaving:
    def setup(self):
        self.old_makedirs = os.makedirs
        os.makedirs = lambda v: None

    def teardown(self):
        os.makedirs = self.old_makedirs
        del self.old_makedirs

    def test_filestorage_requires_name(self):
        uset = UploadSet('files')
        # no name passed in here
        tfs = TestingFileStorage()
        with pytest.raises(ValueError):
            uset.save(tfs)

    def test_saved(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo.txt')
        res = uset.save(tfs)
        assert res == 'foo.txt'
        assert tfs.saved == '/uploads/foo.txt'

    def test_save_folders(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo.txt')
        res = uset.save(tfs, folder='someguy')
        assert res == 'someguy/foo.txt'
        assert tfs.saved == '/uploads/someguy/foo.txt'

    def test_save_named(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo.txt')
        res = uset.save(tfs, name='file_123.txt')
        assert res == 'file_123.txt'
        assert tfs.saved == '/uploads/file_123.txt'

    def test_save_namedext(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='boat.jpg')
        res = uset.save(tfs, name='photo_123.')
        assert res == 'photo_123.jpg'
        assert tfs.saved == '/uploads/photo_123.jpg'

    def test_folder_namedext(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='boat.jpg')
        res = uset.save(tfs, folder='someguy', name='photo_123.')
        assert res == 'someguy/photo_123.jpg'
        assert tfs.saved == '/uploads/someguy/photo_123.jpg'

    def test_implicit_folder(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='boat.jpg')
        res = uset.save(tfs, name='someguy/photo_123.')
        assert res == 'someguy/photo_123.jpg'
        assert tfs.saved == '/uploads/someguy/photo_123.jpg'

    def test_secured_filename(self):
        uset = UploadSet('files', ALL)
        uset._config = Config('/uploads')
        tfs1 = TestingFileStorage(filename='/etc/passwd')
        tfs2 = TestingFileStorage(filename='../../my_app.wsgi')
        res1 = uset.save(tfs1)
        assert res1 == 'etc_passwd'
        assert tfs1.saved == '/uploads/etc_passwd'
        res2 = uset.save(tfs2)
        assert res2 == 'my_app.wsgi'
        assert tfs2.saved == '/uploads/my_app.wsgi'

    def test_storage_is_not_a_werkzeug_datastructure(self):
        """UploadSet.save needs a valid FileStorage object.

        When something different is passed in, a TypeError gets raised.
        """
        uset = UploadSet('files', ALL)
        uset._config = Config('/uploads')
        non_storage = 'this is no werkzeug.datastructure.FileStorage'

        with pytest.raises(TypeError):
            uset.save(non_storage)

    def test_file_not_allowed(self):
        """Raise UploadNotAllowed for not allowed file extensions."""
        uset = UploadSet('files', ('png'))
        uset._config = Config('/uploads')
        testing_filestorage = TestingFileStorage(filename='picture.gif')
        with pytest.raises(UploadNotAllowed):
            uset.save(testing_filestorage)


class TestConflictResolution:
    def setup(self):
        self.extant_files = []
        self.old_exists = os.path.exists
        os.path.exists = self.exists
        self.old_makedirs = os.makedirs
        os.makedirs = lambda v: None

    def teardown(self):
        os.path.exists = self.old_exists
        del self.extant_files, self.old_exists
        os.makedirs = self.old_makedirs
        del self.old_makedirs

    def extant(self, *files):
        self.extant_files.extend(files)

    def exists(self, file_name):
        return file_name in self.extant_files

    def test_self(self):
        assert not os.path.exists('/uploads/foo.txt')
        self.extant('/uploads/foo.txt')
        assert os.path.exists('/uploads/foo.txt')

    def test_conflict(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo.txt')
        self.extant('/uploads/foo.txt')
        res = uset.save(tfs)
        assert res == 'foo_1.txt'

    def test_multi_conflict(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo.txt')
        self.extant('/uploads/foo.txt',
                    *('/uploads/foo_%d.txt' % n for n in range(1, 6)))
        res = uset.save(tfs)
        assert res == 'foo_6.txt'

    def test_conflict_without_extension(self):
        uset = UploadSet('files', extensions=(''))
        uset._config = Config('/uploads')
        tfs = TestingFileStorage(filename='foo')
        self.extant('/uploads/foo')
        res = uset.save(tfs)
        assert res == 'foo_1'


class TestPathsAndURLs:
    def test_path(self):
        uset = UploadSet('files')
        uset._config = Config('/uploads')
        assert uset.path('foo.txt') == '/uploads/foo.txt'
        assert uset.path('someguy/foo.txt') == '/uploads/someguy/foo.txt'
        assert (uset.path('foo.txt', folder='someguy') ==
                '/uploads/someguy/foo.txt')
        assert (uset.path('foo/bar.txt', folder='someguy') ==
                '/uploads/someguy/foo/bar.txt')

    def test_url_generated(self):
        app = Flask(__name__)
        app.config.update(
            UPLOADED_FILES_DEST='/uploads'
        )
        app.config["UPLOADS_AUTOSERVE"] = True
        uset = UploadSet('files')
        configure_uploads(app, uset)
        with app.test_request_context():
            url = uset.url('foo.txt')
            gen = url_for('_uploads.uploaded_file', setname='files',
                          filename='foo.txt', _external=True)
            assert url == gen

    def test_url_based(self):
        app = Flask(__name__)
        app.config.update(
            UPLOADED_FILES_DEST='/uploads',
            UPLOADED_FILES_URL='http://localhost:5001/'
        )
        uset = UploadSet('files')
        configure_uploads(app, uset)
        with app.test_request_context():
            url = uset.url('foo.txt')
            assert url == 'http://localhost:5001/foo.txt'
        assert '_uploads' not in app.blueprints


def test_configure_for_set_throws_runtimeerror():
    """when there is no destination for an UploadSet"""
    upload_set = UploadSet("files")
    app = Flask(__name__)
    with pytest.raises(RuntimeError):
        config_for_set(upload_set, app)
